/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef IDIOSKOPOSUTILITY_H
#define IDIOSKOPOSUTILITY_H

#include <idioskopos/property.h>
#include <idioskopos/error.h>

namespace Idioskopos {

/**
 * Type checked dynamic cast from a generic Idioskopos::Object to a Property
 */
template <typename T>
inline
Property<T>& property_cast(Object& pb) throw (error::bad_cast) {
    if (pb.type() != typeid(T))
        throw error::cast::wrong_property_type();
    return dynamic_cast<Property<T>&>(pb);
}

/**
 * Type checked dynamic cast from a generic Idioskopos::Object pointer to a Property
 */
template <typename T>
inline
Property<T>& property_cast(Object* pb) throw (error::bad_cast) {
    if (pb == NULL)
        throw error::cast::null_pointer();
    if (pb->type() != typeid(T))
        throw  error::cast::wrong_property_type();
    return *(dynamic_cast<Property<T>*>(pb));
}

/**
 * Given an Introspectable and a property name, performs a type checked
 * dynamic cast to a Property
 */
template <typename T>
inline
Property<T>& property(Introspectable& obj, const Glib::ustring name) throw (error::bad_cast) {
    Object* pb = obj.get_property(name);
    if (pb == NULL)
        throw  error::cast::null_pointer();
    return property_cast<T>(*pb);
}

}

#endif
